/*** Ezt Init:begin ***/


/* #define the following:

EZT_NAMESPACE	Main namespace where lives the extension. Example: "::myns"
EZT_ENSEMBLE	Name of ensemble command that lives in EZT_NAMESPACE. Example: "mycmd"
EZT_COMMAND	Name of ensemble subcommand. Example: "mysubcmd". Usually same as ensemble?

Optional:
EZT_ENSEMBLE_COMMAND	Name of ensemble subcommand if ensemble subcommand is itself an ensemble. Example: "mysubcommand"
EZT_COMMAND_CLIENTDATA	#define if subcommands use ClientData.
EZT_COMMAND_PROC	Only a single command that lives in the ensemble. Example: Myext_MyCmd

Init:
EZT_INIT	Extension's init function. Example: Myext_Init
EZT_SAFE_INIT	Extension's safe init function. Example: Myext_SafeInit

Optional init:
EZT_EXT_INIT	Extension's init function. Called from init with interp. Example: Myext_ExtInit
EZT_PKGINIT	Extension's package init function. Called from package init with interp and namespace. Example: Myext_PkgInit

*/

/* Fully-qualified ensemble command */
#define EZT_FQ_ENSEMBLE EZT_NAMESPACE "::" EZT_ENSEMBLE

static int
Ezt_PackageInit (Tcl_Interp *interp) {
	Tcl_Namespace *ns;
#ifndef EZT_COMMAND_PROC
	ClientData clientData = NULL;
	const Ezt_Cmd *cmd;
	Tcl_Obj *nameObj;
#endif

	/* Main namespace */
	if ((ns = Tcl_FindNamespace(interp, EZT_NAMESPACE, NULL, 0)) == NULL) {
		if ((ns = Tcl_CreateNamespace(interp, EZT_NAMESPACE, NULL, NULL)) == NULL) { return TCL_ERROR; }
	}

	/* Export only the ensemble name. Flag 0 means don't reset the export list. */
	if (Tcl_Export (interp, ns, EZT_ENSEMBLE, 0) != TCL_OK) { return TCL_ERROR; }

	/* Ensemble or command namespace */
	if ((ns = Tcl_FindNamespace (interp, EZT_FQ_ENSEMBLE, NULL, 0)) == NULL) {
		if ((ns = Tcl_CreateNamespace (interp, EZT_FQ_ENSEMBLE, NULL, NULL)) == NULL) { return TCL_ERROR; }
	}

#ifdef EZT_COMMAND_PROC
	/* Only a single command that lives in the ensemble */
	if (Tcl_CreateObjCommand(interp, EZT_FQ_ENSEMBLE "::" EZT_COMMAND, EZT_COMMAND_PROC, NULL, NULL) == NULL) { return TCL_ERROR; }
#else

#  ifdef EZT_ENSEMBLE_COMMAND
#    define EZT_CMDFMT EZT_FQ_ENSEMBLE "::" EZT_ENSEMBLE_COMMAND
	/* Main ensemble subcommand is itself an ensemble */
	if ((ns = Tcl_CreateNamespace (interp, EZT_CMDFMT, NULL, NULL)) == NULL) { return TCL_ERROR; }
#  else
#    define EZT_CMDFMT EZT_FQ_ENSEMBLE
#  endif

	/* All subcommands for ensemble */
	for (cmd = &Ezt_Cmds[0]; cmd->name != NULL; cmd++) {
#  ifdef EZT_COMMAND_CLIENTDATA
		clientData = cmd->clientData;
#  endif
		nameObj = Tcl_ObjPrintf(EZT_CMDFMT "::" "%s", cmd->name);
		Tcl_IncrRefCount(nameObj);
		if (Tcl_CreateObjCommand(interp, Tcl_GetString(nameObj), cmd->proc, clientData, NULL) == NULL) { Tcl_DecrRefCount(nameObj); return TCL_ERROR; }
		Tcl_DecrRefCount(nameObj);
		if (Tcl_Export(interp, ns, cmd->name, 0) != TCL_OK) { return TCL_ERROR; }
	}

#  ifdef EZT_ENSEMBLE_COMMAND
	/* Flag 0 means full name required */
	if (Tcl_CreateEnsemble (interp, EZT_CMDFMT, ns, 0) == NULL) { return TCL_ERROR; }
#  endif
#endif

	/* Ensemble */
	if ((ns = Tcl_FindNamespace (interp, EZT_FQ_ENSEMBLE, NULL, TCL_LEAVE_ERR_MSG)) == NULL) { return TCL_ERROR; }
	if (Tcl_CreateEnsemble (interp, EZT_FQ_ENSEMBLE, ns, TCL_ENSEMBLE_PREFIX) == NULL) { return TCL_ERROR; }
	if (Tcl_Export (interp, ns, EZT_COMMAND, 0) != TCL_OK) { return TCL_ERROR; }

#ifdef EZT_PKGINIT
	if (EZT_PKGINIT (interp, EZT_NAMESPACE) != TCL_OK) { return TCL_ERROR; }
#endif

	return TCL_OK;
}

static int
Ezt_PackageProvide (Tcl_Interp *interp) {
	return Tcl_PkgProvide(interp, PACKAGE_NAME, PACKAGE_VERSION);
}

static int
Ezt_CommonInit (Tcl_Interp *interp) {
	if (Tcl_InitStubs       (interp,        EZT_MIN_TCL_VER, 0) == NULL) { return TCL_ERROR; }
	if (Tcl_PkgRequire      (interp, "Tcl", EZT_MIN_TCL_VER, 0) == NULL) { return TCL_ERROR; }
	return TCL_OK;
}

static int
Ezt_Init (Tcl_Interp *interp) {
	if (Ezt_CommonInit     (interp) != TCL_OK) { return TCL_ERROR; }
#ifdef EZT_EXT_INIT
	if (EZT_EXT_INIT       (interp) != TCL_OK) { return TCL_ERROR; }
#endif
	if (Ezt_PackageInit    (interp) != TCL_OK) { return TCL_ERROR; }
	if (Ezt_PackageProvide (interp) != TCL_OK) { return TCL_ERROR; }
	return TCL_OK;
}

/***/

EXTERN int
EZT_INIT (Tcl_Interp *interp) {
	return Ezt_Init(interp);
}

EXTERN int
EZT_SAFE_INIT (Tcl_Interp *interp) {
	return EZT_INIT(interp);
}

/*** Ezt Init:end ***/
